@@ -1,7 +1,7 @@
 package WWW::PAUSE::Simple;
 
-our $DATE = '2015-09-06'; # DATE
-our $VERSION = '0.25'; # VERSION
+our $DATE = '2015-09-10'; # DATE
+our $VERSION = '0.26'; # VERSION
 
 use 5.010001;
 use strict;
@@ -77,6 +77,19 @@ our %file_opt_arg = (
         'x.name.is_plural' => 1,
         pos => 0,
         greedy => 1,
+        tags => ['category:filtering'],
+    },
+);
+
+our %mod_opt_arg = (
+    modules => {
+        summary => 'Module names/wildcard patterns',
+        'summary.alt.plurality.singular' => 'Module name/wildcard pattern',
+        schema  => ['array*', of=>'str*'],
+        'x.name.is_plural' => 1,
+        pos => 0,
+        greedy => 1,
+        tags => ['category:filtering'],
     },
 );
 
@@ -125,7 +138,7 @@ sub _htres2envres {
 
 $SPEC{upload_files} = {
     v => 1.1,
-    summary => 'Upload file(s) to your PAUSE account',
+    summary => 'Upload file(s)',
     args_rels => {
         choose_one => [qw/delay group_delay/],
     },
@@ -295,7 +308,7 @@ sub upload_files {
 
 $SPEC{list_files} = {
     v => 1.1,
-    summary => 'List files on your PAUSE account',
+    summary => 'List files',
     args => {
         %common_args,
         %detail_l_arg,
@@ -387,7 +400,7 @@ sub list_files {
 
 $SPEC{list_dists} = {
     v => 1.1,
-    summary => 'List distributions on your PAUSE account',
+    summary => 'List distributions',
     description => <<'_',
 
 Distribution names will be extracted from tarball/zip filenames.
@@ -501,7 +514,7 @@ sub list_dists {
 
 $SPEC{delete_old_releases} = {
     v => 1.1,
-    summary => 'Delete older versions of distributions on your PAUSE account',
+    summary => 'Delete older versions of distributions',
     description => <<'_',
 
 Developer releases will not be deleted.
@@ -681,6 +694,7 @@ $SPEC{set_password} = {
     args => {
         %common_args,
     },
+    'x.no_index' => 1,
 };
 sub set_password {
     my %args = @_;
@@ -692,12 +706,101 @@ $SPEC{set_account_info} = {
     args => {
         %common_args,
     },
+    'x.no_index' => 1,
 };
 sub set_account_info {
     my %args = @_;
     [501, "Not yet implemented"];
 }
 
+$SPEC{list_modules} = {
+    v => 1.1,
+    summary => 'List modules (permissions)',
+    args => {
+        %common_args,
+        %detail_l_arg,
+        %mod_opt_arg,
+        type => {
+            summary => 'Only list modules matching certain type',
+            schema => 'str*',
+            tags => ['category:filtering'],
+        },
+    },
+};
+sub list_modules {
+    my %args  = @_;
+    require Regexp::Wildcards;
+    require String::Wildcard::Bash;
+
+    my $q = $args{modules} // [];
+
+    my %post_data = (ACTION=>'peek_perms');
+
+    # optimize: the PAUSE server can do SQL LIKE, if there is only a single
+    # module argument we pass it to server to reduce traffic
+    if (@$q == 1) {
+        $post_data{pause99_peek_perms_by} = 'ml';
+        $post_data{pause99_peek_perms_query} =
+            String::Wildcard::Bash::convert_wildcard_to_sql($q->[0]);
+        $post_data{pause99_peek_perms_sub} = 'Submit';
+    }
+
+    my $httpres = _request(
+        %args,
+        post_data => [\%post_data],
+    );
+
+    return _htres2envres($httpres) unless $httpres->is_success;
+
+    # convert wildcard patterns in arguments to regexp
+    for (@$q) {
+        next unless String::Wildcard::Bash::contains_wildcard($_);
+        my $re = Regexp::Wildcards->new(type=>'unix')->convert($_);
+        $re = qr/\A($re)\z/;
+        $_ = $re;
+    }
+
+    return [543, "Can't scrape list of modules from response",
+            $httpres->content]
+        unless $httpres->content =~ m!<tr><td><b>module</b></td>.+?</tr>(.+?)</table>!s;
+    my $str = $1;
+    my @mods;
+  REC:
+    while ($str =~ m!<tr><td><a[^>]+>(.+?)</a></td>\s*<td><a[^>]+>(.+?)</a></td>\s*<td>(.+?)</td>\s*<td>(.+?)</td>\s*</tr>!gs) {
+        my $rec = {module=>$1, userid=>$2, type=>$3, owner=>$4};
+
+        # filter by requested file/wildcard
+      FILTER_QUERY:
+        {
+            last unless @$q > 1;
+            for (@$q) {
+                if (ref($_) eq 'Regexp') {
+                    last FILTER_QUERY if $rec->{module} =~ $_;
+                } else {
+                    last FILTER_QUERY if $rec->{module} eq $_;
+                }
+            }
+            # nothing matches
+            next REC;
+        }
+
+      FILTER_TYPE:
+        if ($args{type}) {
+            next REC unless $rec->{type} eq $args{type};
+        }
+
+        push @mods, $args{detail} ? $rec : $rec->{module};
+    }
+    my %resmeta;
+    if ($args{detail}) {
+        $resmeta{format_options} = {
+            any => {
+                table_column_orders => [[qw/module userid type owner/]],
+            },
+        };
+    }
+    [200, "OK", \@mods, \%resmeta];
+}
 
 1;
 # ABSTRACT: An API for PAUSE
@@ -714,7 +817,7 @@ WWW::PAUSE::Simple - An API for PAUSE
 
 =head1 VERSION
 
-This document describes version 0.25 of WWW::PAUSE::Simple (from Perl distribution WWW-PAUSE-Simple), released on 2015-09-06.
+This document describes version 0.26 of WWW::PAUSE::Simple (from Perl distribution WWW-PAUSE-Simple), released on 2015-09-10.
 
 =head1 SYNOPSIS
 
@@ -779,7 +882,7 @@ Return value:  (any)
 
 =head2 delete_old_releases(%args) -> [status, msg, result, meta]
 
-Delete older versions of distributions on your PAUSE account.
+Delete older versions of distributions.
 
 Developer releases will not be deleted.
 
@@ -841,7 +944,7 @@ Return value:  (any)
 
 =head2 list_dists(%args) -> [status, msg, result, meta]
 
-List distributions on your PAUSE account.
+List distributions.
 
 Distribution names will be extracted from tarball/zip filenames.
 
@@ -891,7 +994,7 @@ Return value:  (any)
 
 =head2 list_files(%args) -> [status, msg, result, meta]
 
-List files on your PAUSE account.
+List files.
 
 Arguments ('*' denotes required arguments):
 
@@ -931,6 +1034,48 @@ that contains extra information.
 Return value:  (any)
 
 
+=head2 list_modules(%args) -> [status, msg, result, meta]
+
+List modules (permissions).
+
+Arguments ('*' denotes required arguments):
+
+=over 4
+
+=item * B<detail> => I<bool>
+
+Whether to return detailed records.
+
+=item * B<modules> => I<array[str]>
+
+Module names/wildcard patterns.
+
+=item * B<password>* => I<str>
+
+PAUSE password.
+
+=item * B<type> => I<str>
+
+Only list modules matching certain type.
+
+=item * B<username>* => I<str>
+
+PAUSE ID.
+
+=back
+
+Returns an enveloped result (an array).
+
+First element (status) is an integer containing HTTP status code
+(200 means OK, 4xx caller error, 5xx function error). Second element
+(msg) is a string containing error message, or 'OK' if status is
+200. Third element (result) is optional, the actual result. Fourth
+element (meta) is called result metadata and is optional, a hash
+that contains extra information.
+
+Return value:  (any)
+
+
 =head2 reindex_files(%args) -> [status, msg, result, meta]
 
 Force reindexing.
@@ -978,62 +1123,6 @@ that contains extra information.
 Return value:  (any)
 
 
-=head2 set_account_info(%args) -> [status, msg, result, meta]
-
-Arguments ('*' denotes required arguments):
-
-=over 4
-
-=item * B<password>* => I<str>
-
-PAUSE password.
-
-=item * B<username>* => I<str>
-
-PAUSE ID.
-
-=back
-
-Returns an enveloped result (an array).
-
-First element (status) is an integer containing HTTP status code
-(200 means OK, 4xx caller error, 5xx function error). Second element
-(msg) is a string containing error message, or 'OK' if status is
-200. Third element (result) is optional, the actual result. Fourth
-element (meta) is called result metadata and is optional, a hash
-that contains extra information.
-
-Return value:  (any)
-
-
-=head2 set_password(%args) -> [status, msg, result, meta]
-
-Arguments ('*' denotes required arguments):
-
-=over 4
-
-=item * B<password>* => I<str>
-
-PAUSE password.
-
-=item * B<username>* => I<str>
-
-PAUSE ID.
-
-=back
-
-Returns an enveloped result (an array).
-
-First element (status) is an integer containing HTTP status code
-(200 means OK, 4xx caller error, 5xx function error). Second element
-(msg) is a string containing error message, or 'OK' if status is
-200. Third element (result) is optional, the actual result. Fourth
-element (meta) is called result metadata and is optional, a hash
-that contains extra information.
-
-Return value:  (any)
-
-
 =head2 undelete_files(%args) -> [status, msg, result, meta]
 
 Undelete files.
@@ -1087,7 +1176,7 @@ Return value:  (any)
 
 =head2 upload_files(%args) -> [status, msg, result, meta]
 
-Upload file(s) to your PAUSE account.
+Upload file(s).
 
 This function supports dry-run operation.
 
@@ -1164,7 +1253,7 @@ that contains extra information.
 
 Return value:  (any)
 
-=for Pod::Coverage ^(upload_file)$
+=for Pod::Coverage ^(upload_file|set_account_info|set_password)$
 
 =head1 SEE ALSO
 
